#include <linux/module.h>
#include <linux/kernel.h>
#include <linux/backlight.h>
#include <linux/device.h>
#include <linux/dmi.h>

#define BACKLIGHT_OFFSET 0xb9
#define KBBACKLIGHT_OFFSET 0x7a
#define AUTOBACKLIGHT_OFFSET 0x5c

extern int ec_read(u8 addr, u8 *val); 
extern int ec_write(u8 addr, u8 val);

static int n70z_get_brightness(struct backlight_device *);
static int n70z_set_brightness(struct backlight_device *);

/* Backlight device object */
static struct backlight_device * n70z_backlight_dev = NULL;
/* Backlight device operations table object */
static struct backlight_ops n70z_backlight_ops = 
{
	.get_brightness = n70z_get_brightness,
	.update_status = n70z_set_brightness,
};



/* Keyboard backlight status show */
static ssize_t n70z_kbbacklight_show(struct device *dev,
							struct device_attribute *attr,
						    char *buf)
{
	u8 tmp;
	int ret = 0;
	ret = ec_read(KBBACKLIGHT_OFFSET,&tmp);
	if(ret)
	{
		pr_err("Get keyboard brightness error.\n");
		return ret;
	}
	pr_debug("keyboard brightness = %x.\n",tmp);
	return  scnprintf(buf, sizeof(buf), "%d\n", tmp);
}
static DEVICE_ATTR_RO(n70z_kbbacklight);


/* Auto set  backlight status */
static ssize_t auto_backlight_status_show(struct device *dev,
					struct device_attribute *attr, char *buf)
{
	u8 tmp;
	int ret = 0;
	ret = ec_read(AUTOBACKLIGHT_OFFSET,&tmp);
	if(ret)
	{
		pr_err("Get auto backlight status error.\n");
		return ret;
	}
	tmp &= 0x1;
	pr_debug("Auto backlight stauts = %x.\n",tmp);
	return  scnprintf(buf, sizeof(buf), "%d\n", tmp);
}

static ssize_t auto_backlight_status_store(struct device *dev,
					struct device_attribute *attr, 
					const char *buf,size_t count)
{
	u8 ec_value,user_value;
	char *after;
	int ret = 0;
	user_value = simple_strtoul(buf,&after,10); 
	ret = ec_read(AUTOBACKLIGHT_OFFSET,&ec_value);
	if(ret)
	{
		pr_err("Get auto backlight status error.\n");
		return ret;
	}
	if(user_value) 
		ec_value |= 0x1;
	else 
		ec_value &= ~0x1;

	ret = ec_write(AUTOBACKLIGHT_OFFSET,ec_value);
	if(ret)
	{
		pr_err("Set auto backlight status error.\n");
		return ret;
	}
	pr_debug("Set auto backlight stauts = %x.\n",ec_value&0x1);
	return  sizeof(buf);
}

static DEVICE_ATTR(auto_backlight_status, S_IRUGO | S_IWUSR, auto_backlight_status_show, auto_backlight_status_store);

/* Backlight device set brightness handler */
static int n70z_set_brightness(struct backlight_device * pdev)
{
	int ret;
	unsigned int level = 0;
	pr_info("brightness set.\n");
	level = pdev->props.brightness;
	pr_debug("brightness level = %x.",level);
	
	if(level == 0)
		level = 1;
	if(level > 100)
		level = 100;

	ret = ec_write(BACKLIGHT_OFFSET,level);
	if(ret)
	{
		pr_err("Set brightness error.\n");
		return ret;
	}
	ret = 0;
	return ret;
}


/* Backlight device get brightness handler */
static int n70z_get_brightness(struct backlight_device * pdev)
{
	int ret; 
	u8 tmp;
	pr_info("Brightness get.\n");
	ret = ec_read(BACKLIGHT_OFFSET,&tmp);
	if(ret)
	{
		pr_err("Get brightness error.\n");
		return ret;
	}
	pr_info("Brightness vale = %x.\n",tmp);
	ret = tmp;	
	return ret;
}


static int __init n70z_backlight_init(void)
{
	int ret = 0;
	u8 brightness_default;
	u8 osfit;
	pr_info("Lenovo n70z_backlight init.\n");
	printk("%s\n",dmi_get_system_info(DMI_BOARD_NAME));
	/*
	if (!strstr( dmi_get_system_info(DMI_BOARD_NAME), "LXKT-ZXE-N70"))
	{
		pr_info("It is not N70.");
		ret = -1;
		goto fail_backlight_device_register;
	}
	*/

	/*Set os fit value. 0xa3 value |= 0x02 then write to 0xa3*/	
	
	ret = ec_read(0xa3, &osfit);
	if(ret)
	{
		pr_err("Get osfit value of 0xa3 error.\n");
		return ret;
	}
	
	osfit |= 0x02;
	ret = ec_write(0xa3, osfit);
	if(ret)
	{
		pr_err("Write osfit value to 0xa3 error.\n");
		return ret;
	}
	//////////////////////////

	/* Regist backlight */
	n70z_backlight_dev = backlight_device_register("n70z", NULL, NULL,
			&n70z_backlight_ops, NULL);
	if (IS_ERR(n70z_backlight_dev)) {
		pr_err("n70z_backlight init error.\n");
		ret = PTR_ERR(n70z_backlight_dev);
		goto fail_backlight_device_register;
	}

	n70z_backlight_dev->props.max_brightness = 100;

#if 0
	/* Get default brightness */
	ret = ec_read(BACKLIGHT_OFFSET,&brightness_default);
	if(ret)
	{
		pr_info("can't get default brightness.\n");
		goto fail_get_set_default_brightness;
	} 
	if(brightness_default == 0 || brightness_default > 100) 
		brightness_default = 100;
#else
	brightness_default = 100;
#endif

	ret = ec_write(BACKLIGHT_OFFSET,brightness_default);
	if(ret)
	{
		pr_info("can't set default brightness,module exit.\n");
		goto fail_get_set_default_brightness;
	} 

	n70z_backlight_dev->props.brightness = brightness_default;
	backlight_update_status(n70z_backlight_dev);
	device_create_file(&n70z_backlight_dev->dev, &dev_attr_n70z_kbbacklight);
	device_create_file(&n70z_backlight_dev->dev, &dev_attr_auto_backlight_status);
	return ret;

fail_get_set_default_brightness:
	backlight_device_unregister(n70z_backlight_dev);
fail_backlight_device_register:
	return ret;	
}

static void __exit n70z_backlight_exit(void)
{
	pr_info("n70z_backlight exit.\n");
	device_remove_file(&n70z_backlight_dev->dev, &dev_attr_n70z_kbbacklight);
	device_remove_file(&n70z_backlight_dev->dev, &dev_attr_auto_backlight_status);
	backlight_device_unregister(n70z_backlight_dev);
}



module_init(n70z_backlight_init)
module_exit(n70z_backlight_exit)
MODULE_AUTHOR("support <support@nfschina.com>");
MODULE_DESCRIPTION("n70z backlight Driver");
MODULE_LICENSE("GPL");
